/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/log.h"
#include "ns3/topology.h"
#include "ns3/ipv4-address-allocator.h"
#include "ns3/ipv4-address-manager.h"
#include "ns3/ipv4-protocol-manager.h"
#include "ns3/udp-echo-client.h"
#include "ns3/udp-echo-server.h"
#include "ns3/uinteger.h"
#include "ns3/simulator.h"
#include "ns3/nstime.h"
#include "ns3/default-value.h"
#include "ns3/ascii-trace.h"

#include "ns3/infra-wifi-netdevice-manager.h"

NS_LOG_COMPONENT_DEFINE ("InfraUdpEchoSimulation");

using namespace ns3;

class MyApplicationManager : public ApplicationManager
{
private:
  void CreateApplications (Ptr<Node> node);
};

  void
MyApplicationManager::CreateApplications (Ptr<Node> node)
{
  Ptr<UdpEchoServer> server = CreateObjectWith<UdpEchoServer> (
    "Node", node,
    "Port", Uinteger (7));

  server->Start(Seconds(1.));
  server->Stop (Seconds(10.));
}

int 
main (int argc, char *argv[])
{
  NS_LOG_INFO ("INFRA UDP Echo Simulation");

  DefaultValue::Bind ("WifiRtsCtsThreshold", "0");
  DefaultValue::Bind ("WifiFragmentationThreshold", "2200");
  DefaultValue::Bind ("WifiRateControlAlgorithm", "Aarf");

  Ptr<Ipv4AddressAllocator> ipAddrs = CreateObject<Ipv4AddressAllocator> ();
  ipAddrs->Init ("192.168.0.0", "255.255.255.0", "0.0.0.1");

  Ptr<NetworkManager> net = CreateObject<NetworkManager> ();
  net->SetNetDeviceManager (CreateObject<InfraWifiNetDeviceManager> (
    "demo", 0));  // the zeroth netdevice created will be the access point.
  net->SetProtocolManager (CreateObject<Ipv4ProtocolManager> ());
  net->SetAddressManager (CreateObject<Ipv4AddressManager> (ipAddrs));
  net->SetApplicationManager (CreateObject<MyApplicationManager> ());

  net->Create (10);
  net->Configure ();

  Ptr<UdpEchoClient> client = CreateObjectWith<UdpEchoClient> (
    "Node", net->GetNodeManager() ->GetNode (9), 
    "RemoteIpv4", Ipv4Address ("10.1.0.3"),
    "RemotePort", Uinteger (7), 
    "MaxPackets", Uinteger (1), 
    "Interval", Seconds(1.), 
    "PacketSize", Uinteger (1024));

  client->Start(Seconds(2.));
  client->Stop (Seconds(10.));

  AsciiTrace asciitrace ("infra-echo-topology.tr");
  asciitrace.TraceAllQueues ();

  Simulator::Run ();
  Simulator::Destroy ();
}
