/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/topology.h"
#include "ns3/ipv4-address-allocator.h"
#include "ns3/csma-netdevice-manager.h"
#include "ns3/ipv4-address-manager.h"
#include "ns3/ipv4-protocol-manager.h"

#include "ns3/udp-echo-server.h"
#include "ns3/uinteger.h"

#include "ns3/simulator.h"
#include "ns3/nstime.h"

using namespace ns3;

class MyApplicationManager : public ApplicationManager
{
private:
  void CreateApplications (Ptr<Node> node);
};

  void
MyApplicationManager::CreateApplications (Ptr<Node> node)
{
  Ptr<UdpEchoServer> server = CreateObjectWith<UdpEchoServer> (
    "Node", node, 
    "Port", Uinteger (7));

  node->AddApplication (server);
}

  int 
main (int argc, char *argv[])
{
  //
  // Create a 1000 node network.  The NetworkManager class orchestrates all
  // of the work based on managers we assign to perform sub-tasks.  We use
  // the predefined CSMA version of the net device manager to create a CSMA
  // channel and enough net devices to build the network.  We use the
  // predefined Ipv4 protocol manager to install the Ipv4 protocols on all 
  // of the nodes created.  We Use the predefined Ipv4 version of the address
  // manager to assign addresses to the net devices in all of the nodes.  We
  // provide our own version of the application manager that we created above
  // to install an echo server application on all of the nodes.
  //
  Ptr<Ipv4AddressAllocator> ipAddrs = CreateObject<Ipv4AddressAllocator> ();
  ipAddrs->Init ("192.168.0.0", "255.255.255.0", "0.0.0.1");

  Ptr<NetworkManager> net = CreateObject<NetworkManager> ();
  net->SetNetDeviceManager (CreateObject<CsmaNetDeviceManager> ());
  net->SetProtocolManager (CreateObject<Ipv4ProtocolManager> ());
  net->SetAddressManager (CreateObject<Ipv4AddressManager> (ipAddrs));
  net->SetApplicationManager (CreateObject<MyApplicationManager> ());

  net->Create (10);

  return 0;
}
