/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/log.h"
#include "ns3/wifi-channel.h"
#include "ns3/wifi-net-device.h"
#include "ns3/wifi-mac.h"
#include "ns3/wifi-phy.h"
#include "ns3/wifi-remote-station-manager.h"
#include "ns3/propagation-delay-model.h"
#include "ns3/propagation-loss-model.h"
#include "adhoc-wifi-netdevice-helper.h"

NS_LOG_COMPONENT_DEFINE ("AdhocWifiNetDeviceHelper");

namespace ns3 {

AdhocWifiNetDeviceHelper::AdhocWifiNetDeviceHelper ()
{
  NS_LOG_FUNCTION;
  m_stationManager.SetTypeId ("ArfWifiManager");
  m_phy.SetTypeId ("WifiPhy");
  m_mac.SetTypeId ("AdhocWifiMac");
}

AdhocWifiNetDeviceHelper::~AdhocWifiNetDeviceHelper ()
{
  NS_LOG_FUNCTION;
}

  Ptr<Channel>
AdhocWifiNetDeviceHelper::CreateChannel (void)
{
  NS_LOG_FUNCTION;
  Ptr<WifiChannel> channel = CreateObjectWith<WifiChannel> ();

  channel->SetPropagationDelayModel (
    CreateObjectWith<ConstantSpeedPropagationDelayModel> ());

  Ptr<LogDistancePropagationLossModel> log = 
    CreateObjectWith<LogDistancePropagationLossModel> ();
  log->SetReferenceModel (CreateObjectWith<FriisPropagationLossModel> ());
  channel->SetPropagationLossModel (log);

  return channel;
}

  Ptr<NetDevice>
AdhocWifiNetDeviceHelper::CreateNetDevice (void)
{
  NS_LOG_FUNCTION;
  Ptr<WifiNetDevice> device = CreateObject<WifiNetDevice> ();
  Ptr<WifiRemoteStationManager> manager = 
    m_stationManager.Create<WifiRemoteStationManager> ();
  Ptr<WifiMac> mac = m_mac.Create<WifiMac> ();
  Ptr<WifiPhy> phy = m_phy.Create<WifiPhy> ();
  device->Setup (mac, phy, manager, m_channel->GetObject<WifiChannel> ());
  return device;
}

  void
AdhocWifiNetDeviceHelper::AttachDevice (Ptr<NetDevice> device)
{
  NS_LOG_FUNCTION;
  NS_ASSERT_MSG (0, "AttachDevice done as a side-effect of CreateNetDevice");
}

} // namespace ns3
