/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/log.h"
#include "ns3/queue.h"
#include "ns3/drop-tail-queue.h"
#include "ns3/csma-channel.h"
#include "ns3/csma-net-device.h"
#include "ns3/mac48-address.h"
#include "csma-netdevice-helper.h"

NS_LOG_COMPONENT_DEFINE ("CsmaNetDeviceHelper");

namespace ns3 {

CsmaNetDeviceHelper::CsmaNetDeviceHelper ()
{
  NS_LOG_FUNCTION;
}

CsmaNetDeviceHelper::~CsmaNetDeviceHelper ()
{
  NS_LOG_FUNCTION;
}

  Ptr<Channel>
CsmaNetDeviceHelper::CreateChannel (void)
{
  Ptr<CsmaChannel> ch = CreateObject<CsmaChannel> ();
  return ch;
}

  Ptr<NetDevice>
CsmaNetDeviceHelper::CreateNetDevice (void)
{
  Ptr<CsmaNetDevice> nd = CreateObject<CsmaNetDevice> ();
  Ptr<Queue> q = CreateObjectWith<DropTailQueue> ();
  nd->SetAttribute ("Address", Mac48Address::Allocate());
  nd->AddQueue(q);
  AttachDevice (nd);
  return nd;
}

  void
CsmaNetDeviceHelper::AttachDevice (Ptr<NetDevice> device)
{
  Ptr<CsmaNetDevice> nd = device->GetObject<CsmaNetDevice> ();
  nd->Attach (m_channel->GetObject<CsmaChannel> ());
}

} // namespace ns3
