/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __HELPER_H__
#define __HELPER_H__

#include <vector>
#include "ns3/ptr.h"
#include "ns3/node.h"
#include "ns3/node-container.h"
#include "ns3/channel.h"
#include "ns3/net-device.h"

namespace ns3 {

/**
 * \ingroup Helper
 *
 * \section Helper Overview
 *
 */

class NodeContainer_ : public Object
{
public:
  NodeContainer_ ();
  virtual ~NodeContainer_ ();

  void Create (uint32_t n);
  void Add (Ptr<Node> node);
  bool NodeIsExternal (Ptr<Node> node) const;

  uint32_t GetNNodes (void) const;
  Ptr<Node> GetNode (uint32_t index) const;

protected:
  virtual void PreCreate (void);
  virtual void PostCreate (void);
  virtual Ptr<Node> CreateNode (void);

private:
  typedef std::vector<std::pair<Ptr<Node>, bool> > VectorOfNodePair_t;
  VectorOfNodePair_t m_nodes;
};

class NetDeviceHelper : public Object
{
public:
  NetDeviceHelper ();
  virtual ~NetDeviceHelper ();

  virtual void Create (uint32_t n);
  virtual void Install (NodeContainer &nodes);

  virtual uint32_t GetNDevices (void) const;
  virtual Ptr<NetDevice> GetDevice (uint32_t index) const;

  virtual Ptr<Channel> GetChannel (void) const;

  virtual void PreCreate (void);
  virtual void PostCreate (void);

  virtual void PreInstall (void);
  virtual void PostInstall (void);

  virtual Ptr<NetDevice> CreateNetDevice (void);
  virtual Ptr<Channel> CreateChannel (void);
  virtual void AttachNetDevice (Ptr<NetDevice> device);

  Ptr<Channel> m_channel;

  void Connect (Ptr<NetDevice> netDevice);

  typedef std::vector<Ptr<NetDevice> > VectorOfNetDevice_t;
  VectorOfNetDevice_t m_devices;
};

class ProtocolHelper : public Object
{
public:
  ProtocolHelper ();
  virtual ~ProtocolHelper ();

  void Install (NodeContainer &nodes);

  virtual void PreInstall (void);
  virtual void PostInstall (void);

  virtual void CreateProtocols (Ptr<Node> node);
};

class AddressHelper : public Object
{
public:
  AddressHelper ();
  virtual ~AddressHelper ();

  void Install (NetDeviceHelper &devices);

  virtual void PreInstall (void);
  virtual void PostInstall (void);

  virtual void AssignAddress (Ptr<NetDevice> device);
};

class ApplicationHelper : public Object
{
public:
  ApplicationHelper ();
  virtual ~ApplicationHelper ();

  void Install (NodeContainer &nodes);

  virtual void PreInstall (void);
  virtual void PostInstall (void);

  virtual void CreateApplications (Ptr<Node> node);
};

} // namespace ns3

#endif //__HELPER_H__
