/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/log.h"
#include "ns3/queue.h"
#include "ns3/drop-tail-queue.h"
#include "ns3/csma-channel.h"
#include "ns3/csma-net-device.h"

#include "csma-netdevice-manager.h"

NS_LOG_COMPONENT_DEFINE ("CsmaNetDeviceManager");

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (CsmaNetDeviceManager);

  TypeId 
CsmaNetDeviceManager::GetTypeId (void)
{
  static TypeId tid = TypeId ("CsmaNetDeviceManager")
    .SetParent<NetDeviceManager> ()
    .AddAttribute ("DataRate", "Data rate of channel and net device",
       DataRate (10000000),
       MakeDataRateAccessor (&CsmaNetDeviceManager::m_bps),
       MakeDataRateChecker ())
    .AddAttribute ("Delay", "Speed of light delay of channel",
       Seconds (0.000000333),
       MakeTimeAccessor (&CsmaNetDeviceManager::m_delay),
       MakeTimeChecker ())
    .AddAttribute ("RxErrorModel", "Receive error model",
       Ptr<ErrorModel> (0),
       MakePtrAccessor (&CsmaNetDeviceManager::m_receiveErrorModel),
       MakePtrChecker<ErrorModel> ())
    .AddAttribute ("TxQueue", "Transmit queue of device",
       Ptr<Queue> (0),
       MakePtrAccessor (&CsmaNetDeviceManager::m_queue),
       MakePtrChecker<Queue> ());
  return tid;
}

CsmaNetDeviceManager::CsmaNetDeviceManager ()
{
  NS_LOG_FUNCTION;
}

CsmaNetDeviceManager::~CsmaNetDeviceManager ()
{
  NS_LOG_FUNCTION;
}

  Ptr<Channel>
CsmaNetDeviceManager::CreateChannel (void)
{
  DataRate dataRate = GetAttribute ("DataRate");
  Time delay = GetAttribute ("Delay");
  Ptr<CsmaChannel> ch = CreateObject<CsmaChannel> (dataRate, delay);
  return ch;
}

  Ptr<NetDevice>
CsmaNetDeviceManager::CreateNetDevice (void)
{
  Ptr<CsmaNetDevice> nd = CreateObject<CsmaNetDevice> ();
  nd->SetAttribute ("Address", Mac48Address::Allocate ());
  nd->SetAttribute ("DataRate", GetAttribute ("DataRate"));
  Ptr<Queue> q = CreateObjectWith<DropTailQueue> ();
  nd->AddQueue(q);
  AttachDevice (nd);

  return nd;
}

  void
CsmaNetDeviceManager::AttachDevice (Ptr<NetDevice> device)
{
  Ptr<CsmaNetDevice> nd = device->GetObject<CsmaNetDevice> ();
  nd->Attach (m_channel->GetObject<CsmaChannel> ());
}

} // namespace ns3
