/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __CSMA_NETDEVICE_MANAGER_H__
#define __CSMA_NETDEVICE_MANAGER_H__

#include "topology.h"
#include "ns3/nstime.h"
#include "ns3/data-rate.h"
#include "ns3/error-model.h"
#include "ns3/queue.h"
#include "ns3/ptr.h"

namespace ns3 {

/**
 * \ingroup Topology
 *
 * \section CsmaNetworkManager
 *
 * A network has a channel and a number of nodes connected to the channel via
 * appropriate net devices.
 */

class CsmaNetDeviceManager : public NetDeviceManager
{
public:
  static TypeId GetTypeId (void);

  CsmaNetDeviceManager ();
  virtual ~CsmaNetDeviceManager ();

  Ptr<NetDevice> CreateNetDevice (void);
  Ptr<Channel> CreateChannel (void);
  void AttachDevice (Ptr<NetDevice> device);
protected:
  DataRate m_bps;
  Time m_delay;
  Ptr<ErrorModel> m_receiveErrorModel;
  Ptr<Queue> m_queue;
};

} // namespace ns3

#endif //__CSMA_NETDEVICE_MANAGER_H__

