/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/log.h"
#include "ipv4-address-generator.h"
#include "ipv4-address-allocator.h"

NS_LOG_COMPONENT_DEFINE ("Ipv4AddressAllocator");

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (Ipv4AddressAllocator);

  TypeId 
Ipv4AddressAllocator::GetTypeId (void)
{
  static TypeId tid = TypeId ("Ipv4AddressAllocator")
    .SetParent<Object> ()
    .AddConstructor<Ipv4AddressAllocator> ();
  return tid;
}

Ipv4AddressAllocator::Ipv4AddressAllocator ()
{
  NS_LOG_FUNCTION;
}

Ipv4AddressAllocator::~Ipv4AddressAllocator ()
{
  NS_LOG_FUNCTION;
}

  void 
Ipv4AddressAllocator::Init (
  Ipv4Address prefix, 
  Ipv4Mask mask, 
  Ipv4Address base)
{
  NS_LOG_FUNCTION;
  Ipv4AddressGenerator::SeedNetwork (mask, prefix);
  Ipv4AddressGenerator::SeedAddress (mask, base);
  m_prefix = prefix;
  m_mask = mask;
  m_base = base;
}

  Ipv4Address
Ipv4AddressAllocator::GetNetwork ()
{
  NS_LOG_FUNCTION;
  return m_prefix;
}

  Ipv4Address
Ipv4AddressAllocator::AllocateNetwork (void)
{
  NS_LOG_FUNCTION;
  return AllocateNetwork (m_base);
}

  Ipv4Address
Ipv4AddressAllocator::AllocateNetwork (Ipv4Address base)
{
  NS_LOG_FUNCTION;
  m_base = base;
  Ipv4AddressGenerator::SeedAddress (m_mask, base);
  m_prefix = Ipv4AddressGenerator::AllocateNetwork (m_mask);
  return m_prefix;
}

  Ipv4Address
Ipv4AddressAllocator::AllocateAddress (void)
{
  NS_LOG_FUNCTION;
  return Ipv4AddressGenerator::AllocateAddress (m_mask, m_prefix);
}

  Ipv4Address
Ipv4AddressAllocator::GetAddress (void)
{
  NS_LOG_FUNCTION;
  return Ipv4AddressGenerator::GetAddress (m_mask, m_prefix);
}

  Ipv4Mask
Ipv4AddressAllocator::GetMask (void)
{
  NS_LOG_FUNCTION;
  return m_mask;
}

} // namespace ns3
