/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/log.h"
#include "ns3/ipv4.h"
#include "ipv4-address-manager.h"

NS_LOG_COMPONENT_DEFINE ("Ipv4AddressManager");

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (Ipv4AddressManager);

  TypeId 
Ipv4AddressManager::GetTypeId (void)
{
  static TypeId tid = TypeId ("Ipv4AddressManager")
    .SetParent<Object> ();
  return tid;
}

Ipv4AddressManager::Ipv4AddressManager (Ptr<Ipv4AddressAllocator> ip)
  : m_ip (ip)
{
  NS_LOG_FUNCTION;
}

Ipv4AddressManager::~Ipv4AddressManager ()
{
  NS_LOG_FUNCTION;
}

  void
Ipv4AddressManager::PreInstall (void)
{
  NS_LOG_FUNCTION;
}

  void
Ipv4AddressManager::AssignAddress (Ptr<NetDevice> device)
{
  NS_LOG_FUNCTION;
  Ptr<Node> node = device->GetNode ();
  NS_ASSERT_MSG (node, "Ipv4AddressManager::AssignAddress(): zero node");
  Ptr<Ipv4> ipv4 = node->GetObject<Ipv4> ();
  NS_ASSERT_MSG (ipv4, "Ipv4AddressManager::AssignAddress(): zero ipv4");
  int32_t ifIndex = ipv4->FindInterfaceForDevice (device);
  NS_ASSERT_MSG (ifIndex >= 0, "Ipv4AddressManager::AssignAddress(): "
    "interface index is less than zero");

  Ipv4Address address = m_ip->AllocateAddress ();

  ipv4->SetAddress (ifIndex, address);
  ipv4->SetNetworkMask (ifIndex, m_ip->GetMask ());
  ipv4->SetMetric (ifIndex, 1);
  ipv4->SetUp (ifIndex);
}

} // namespace ns3
