/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2008 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/log.h"
#include "ns3/ipv4.h"
#include "ns3/internet-protocol-stack.h"
#include "ipv4-protocol-manager.h"

NS_LOG_COMPONENT_DEFINE ("Ipv4ProtocolManager");

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (Ipv4ProtocolManager);

  TypeId 
Ipv4ProtocolManager::GetTypeId (void)
{
  static TypeId tid = TypeId ("Ipv4ProtocolManager")
    .SetParent<ProtocolManager> ();

  return tid;
}

Ipv4ProtocolManager::Ipv4ProtocolManager ()
{
  NS_LOG_FUNCTION;
}

Ipv4ProtocolManager::~Ipv4ProtocolManager ()
{
  NS_LOG_FUNCTION;
}

  void
Ipv4ProtocolManager::CreateProtocols (Ptr<Node> node, bool external)
{
  NS_LOG_FUNCTION;
//
// What to do about external nodes.  An external node is one that has been
// previously initialized in some way and added to the node manager that is
// driving this process.  In this case, we are really being asked to make
// sure that the internet protocol stack is present, so we just do that
//
  Ptr<Ipv4> ipv4 = node->GetObject<Ipv4> ();

  if (ipv4 == 0 )
    {
      NS_LOG_INFO ("InstallInternetProtocolStack()");
      InstallInternetProtocolStack (node);
      ipv4 = node->GetObject<Ipv4> ();
    }
//
// In the case of interfaces, we might already have a number of interfaces
// already attached to existing net devices.  Our network manager has added
// one but it won't have an interface yet.  It's our job to find it and add
// the interface.  We'll admit the possibility that more than one device has
// been added, so we'll just make sure they all have interfaces.
//
// XXX what happens when the other device is connected to a non-ipv4 stack?
//
  for (uint32_t i = 0; i < node->GetNDevices (); ++i)
    {
      Ptr<NetDevice> nd = node->GetDevice (i);
      int32_t ifIndex = ipv4->FindInterfaceForDevice(nd);
      if (ifIndex < 0)
        {
          NS_LOG_INFO ("AddInterface() on net device " << nd);
          ipv4->AddInterface (nd);
        }
    }
}

} // namespace ns3
